<?php

namespace Drupal\commerce_funds\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Url;
use Drupal\commerce_payment\Entity\PaymentGateway;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Form to configure the exchange rates.
 */
class ConfigureExchangeRates extends ConfigFormBase {

  /**
   * Defines variables to be used later.
   *
   * @var \Drupal\commerce_store\StoreStorageInterface
   * @var \Drupal\Core\Messenger\MessengerInterface $messenger
   */
  protected $entityTypeManager;
  protected $messenger;

  /**
   * Class constructor.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, MessengerInterface $messenger) {
    $this->entityTypeManager = $entity_type_manager;
    $this->messenger = $messenger;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('messenger')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'commerce_funds_configure_exchange_rates';
  }

  /**
   * {@inheritdoc}
   *
   * Https://www.drupal.org/docs/8/api/form-api/configformbase-with-simple-configuration-api.
   */
  protected function getEditableConfigNames() {
    return [
      'commerce_funds.settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('commerce_funds.settings');

    $enabled_methods = PaymentGateway::loadMultiple() ? PaymentGateway::loadMultiple() : NULL;
    $store = $this->entityTypeManager->getStorage('commerce_store')->loadDefault();
    // If no store set a message.
    if (!$store) {
      $message = $this->t('You haven\'t configured a store yet. Please <a href="@url">configure one</a> before.', [
        '@url' => Url::fromRoute('entity.commerce_store.collection')->toString(),
      ]);
      $this->messenger->addError($message);
      return $form;
    }

    $currencies = $this->entityTypeManager->getStorage('commerce_currency')->loadMultiple();
    $currencyCodesLeft = [];

    // If just one currency.
    if (count($currencies) == 1) {
      $message = $this->t('You just have one currency enabled on your store. <a href="@url">Add more</a> to provide currency conversion.', [
        '@url' => Url::fromRoute('entity.commerce_currency.collection')->toString(),
      ]);
      $this->messenger->addError($message);
      return $form;
    }

    $form['exchange_rates'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Exchange rates'),
      '#collapsible' => FALSE,
    ];

    $exchange_rates = $config->get('exchange_rates') ?: [];

    foreach ($currencies as $currency_left) {
      $currency_code_left = $currency_left->getCurrencyCode();
      foreach ($currencies as $currency_right) {
        $currency_code_right = $currency_right->getCurrencyCode();
        if ($currency_code_left !== $currency_code_right) {
          $form['exchange_rates'][$currency_code_left . '_' . $currency_code_right] = [
            '#type' => 'number',
            '#min' => 0,
            '#title' => $this->t('@currency_left to @currency_right (%)', [
              '@currency_left' => $currency_code_left,
              '@currency_right' => $currency_code_right,
            ]),
            '#default_value' => $exchange_rates ? $exchange_rates[$currency_code_left . '_' . $currency_code_right] : 0,
            '#step' => 0.01,
            '#size' => 10,
            '#maxlength' => 10,
            '#required' => TRUE,
          ];
        }
      }
    }

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $values = $form_state->cleanValues()->getValues();
    $this->config('commerce_funds.settings')
      ->set('exchange_rates', $values)
      ->save();

    parent::submitForm($form, $form_state);
  }

}
