<?php

namespace Drupal\commerce_funds\Plugin\Commerce\PaymentGateway;

use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\commerce_payment\Entity\PaymentInterface;
use Drupal\commerce_payment\Entity\PaymentMethodInterface;
use Drupal\commerce_payment\PaymentMethodTypeManager;
use Drupal\commerce_payment\PaymentTypeManager;
use Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\PaymentGatewayBase;
use Drupal\commerce_price\Calculator;
use Drupal\commerce_funds\Entity\Transaction;
use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;

/**
 * Provides the Balance payment gateway.
 *
 * @CommercePaymentGateway(
 *   id = "funds_balance",
 *   label = @Translation("Funds balance"),
 *   display_label = @Translation("Funds balance"),
 *   forms = {
 *     "add-payment-method" = "Drupal\commerce_funds\PluginForm\Funds\BalanceMethodAddForm",
 *     "edit-payment-method" = "Drupal\commerce_funds\PluginForm\Funds\BalanceMethodAddForm",
 *   },
 *   payment_method_types = {"funds_wallet"},
 * )
 */
class BalanceGateway extends PaymentGatewayBase implements BalanceGatewayInterface {

  use StringTranslationTrait;

  /**
   * {@inheritdoc}
   */
  public function getCurrency() {
    return $this->configuration['currency'];
  }

  /**
   * {@inheritdoc}
   */
  public function getBalanceId() {
    return $this->configuration['balance_id'];
  }

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, EntityTypeManagerInterface $entity_type_manager, PaymentTypeManager $payment_type_manager, PaymentMethodTypeManager $payment_method_type_manager, TimeInterface $time) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $entity_type_manager, $payment_type_manager, $payment_method_type_manager, $time);
  }

  /**
   * {@inheritdoc}
   */
  public function createPayment(PaymentInterface $payment, $catpure = TRUE) {
    parent::assertPaymentState($payment, ['new']);
    $payment_method = $payment->getPaymentMethod();
    parent::assertPaymentMethod($payment_method);

    $this->doPayment($payment_method, $payment);
    $payment->setState('completed');
    $payment->save();
  }

  /**
   * {@inheritdoc}
   */
  public function createPaymentMethod(PaymentMethodInterface $payment_method, array $payment_details) {
    $required_keys = ['balance_id', 'currency'];
    foreach ($required_keys as $required_key) {
      if (empty($payment_details[$required_key])) {
        throw new \InvalidArgumentException(sprintf('$payment_details must contain the %s key.', $required_key));
      }
    }
    $payment_method->balance_id = $payment_details['balance_id'];
    $payment_method->currency = $payment_details['currency'];

    $payment_method->save();
  }

  /**
   * {@inheritdoc}
   */
  public function deletePaymentMethod(PaymentMethodInterface $payment_method) {
    $payment_method->delete();
  }

  /**
   * Performs the payment operation.
   *
   * Remove total price from user balance and update site balance.
   *
   * @see createPaymentMethod()
   */
  protected function doPayment(PaymentMethodInterface $payment_method, PaymentInterface $payment) {
    if ($this->validatePayment($payment_method, $payment)) {
      $transaction = Transaction::create([
        'issuer' => $payment_method->getOwnerId(),
        'recipient' => 1,
        'type' => 'payment',
        'method' => $payment_method->bundle(),
        'brut_amount' => $payment->getAmount()->getNumber(),
        'net_amount' => $payment->getAmount()->getNumber(),
        'fee' => 0,
        'currency' => $payment->getAmount()->getCurrencyCode(),
        'status' => 'Completed',
        'notes' => $this->t('Payment of order <a href="user/@user/orders/@order">@order</a> for an amount of @amount (@currency)', [
          '@user' => $payment_method->getOwnerId(),
          '@order' => $payment->getOrderId(),
          '@amount' => number_format($payment->getAmount()->getNumber(), 2, '.', ','),
          '@currency' => $payment->getAmount()->getCurrencyCode(),
        ]),
      ]);
      $transaction->save();

      \Drupal::service('commerce_funds.transaction_manager')->performTransaction($transaction);
    }
  }

  /**
   * Validate payment before processing.
   *
   * @return bool
   *   Payment is validated or not.
   */
  protected function validatePayment(PaymentMethodInterface $payment_method, PaymentInterface $payment) {
    $order_currency = $payment->getOrder()->getTotalPrice()->getCurrencyCode();
    $balance = \Drupal::service('commerce_funds.transaction_manager')->loadAccountBalance($payment_method->getOwner());

    if ($order_currency !== $payment_method->get('currency')->getValue('target_id')[0]['target_id'] ||
      Calculator::compare($balance[$order_currency], $payment->getOrder()->getTotalPrice()->getNumber(), 2) <= 0) {
      $this->messenger->addError('An error occurred and processing did not complete.');

      return FALSE;
    }

    if ($payment_method->getOwnerId() == 1) {
      $this->messenger->addError('Administrator are not allowed to buy product on their own website.');

      return FALSE;
    }

    return TRUE;
  }

}
